/*=============================================================
　FIX型 電子書籍作成支援：2列の表組からブックマークを作成するスクリプト
===============================================================
● 右列の数値（ノンブル）に前付のページ数を加算し、InDsignのページ番号に揃える
● 空セル、タイトルセル内の改行、右側セルの数値以外文字をエラー表示
● ブックマークを作成（全てレベル１、入れ子は非対応）

配布・改変は自由ですが、販売は禁止します。
本スクリプトに基づいて被ったいかなる損害について、一切責任を負うものではございません。
不具合等が生じてもサポートは一切致しません。
利用した時点で上記に同意したとみなします。

動作確認　InDesign 2024(19.5)
2025/03/25
●エラーマーカーをスウォッチ6番目から、作成するように変更。
●ブックマークパネルを開いたまま実行すると、名称が一部おかしくなる現象が再発したため、実行前に閉じるようにアラートを設置。
●確認のため、表組のテキストフレームを残す仕様に変更。

2023/02/24 Copyright もの書き屋のしっぽ
===============================================================*/
#target InDesign;

main();

// エラー処理
function myerror(mess) {
    if (arguments.length > 0) {
        alert(mess);
    }
    exit();
}

function main() {
    var doc = app.activeDocument;

    // ブックマークパネルが開いているか確認
    try {
        var bookmarksPanel = app.panels.itemByName("ブックマーク");
        if (bookmarksPanel.isValid && bookmarksPanel.visible) {
            alert("エラー：ブックマークパネルが開いています。パネルを閉じてから再実行してください。");
            return; // パネルが開いている場合は処理を終了
        }
    } catch (e) {
        alert("ブックマークパネルを確認できませんでした。エラー：" + e.message);
        return; // パネルの確認中にエラーが発生した場合は終了
    }

    // テキストフレームの選択チェック
    if (doc.selection.length === 0 || !(doc.selection[0] instanceof TextFrame)) {
        myerror("表を含むテキストフレームを選択して下さい");
    }

    var selectedTextFrame = doc.selection[0];
    var mytab = selectedTextFrame.tables.firstItem();

    // テーブルが存在しない場合
    if (!mytab) {
        myerror("テーブルがこのテキストフレームに存在しません。");
    }

    ///// Y100カラーの設定（プロセスカラーでY100を直接設定） /////
    var Y100Color;
    try {
        Y100Color = doc.colors.itemByName("Y100");
        // Y100が存在しない場合、作成する
        if (!Y100Color.isValid) {
            Y100Color = doc.colors.add({name: "Y100", model: ColorModel.PROCESS, space: ColorSpace.CMYK, colorValue: [0, 0, 100, 0]});
        }
    } catch (e) {
        alert("Y100スウォッチの確認・作成中にエラーが発生しました: " + e.message);
        exit();
    }

    ///// 表の背景を紙色にする /////
    var Pcolor = app.activeDocument.swatches.itemByName("Paper"); // デフォルト色
    var rmvCells = mytab.cells.everyItem();
    rmvCells.fillColor = Pcolor;

    ///// セルのエラーチェック /////
    for (var i = 0; i < mytab.rows.length; i++) {
        var checkcell_1 = mytab.rows[i].cells[0];
        var text = checkcell_1.contents;
        var src1 = text.match(/.*[\r\n].*/); // 改行をエラーとする

        if (!text || text == src1) {
            checkcell_1.fillColor = Y100Color; // 空セルや改行があればY100を適用
        }
    }

    for (var i = 0; i < mytab.rows.length; i++) {
        var checkcell_2 = mytab.rows[i].cells[1];
        var ntext = checkcell_2.contents;
        var src2 = ntext.match(/\d*[^0-9^\r^\n]+\d*/); // 数字以外と改行をエラーとする

        if (!ntext || ntext == src2) {
            checkcell_2.fillColor = Y100Color; // 空セルや数値以外があればY100を適用
        }
    }

    for (var i = 0; i < mytab.rows.length; i++) {
        for (var a = 0; a < mytab.rows[i].cells.length; a++) {
            var checkcell = mytab.rows[i].cells[a];
            if (checkcell.fillColor == Y100Color) {
                alert("Error\n空のセル、改行\nノンブルに数値以外の文字\nのどれかが含まれています");
                exit();
            }
        }
    }

    ///// 前付けページ数を加算 /////
    var Frtpage = prompt("前付のページ数を入れてOKを押してください\n見出しが多いと数秒かかります", 0);
    var doc = app.activeDocument;

    if (Frtpage !== null) {
        if (Frtpage >= 0) {
            // ここで既存のブックマーク名を追跡
            var existingNames = {};
            for (var b = 0; b < doc.bookmarks.length; b++) {
                existingNames[doc.bookmarks[b].name] = true;
            }

            for (var i = 0; i < mytab.rows.length; i++) {
                var Pcell = mytab.rows[i].cells[1];
                var Pcnt = Pcell.contents;
                var Pnum = parseInt(Pcnt) + parseInt(Frtpage);
                var GetName = mytab.rows[i].cells[0].contents;

                // ページ番号が範囲内かどうかを確認
                if (Pnum - 1 >= 0 && Pnum - 1 < doc.pages.length) {
                    var page = doc.pages[Pnum - 1]; // インデックスが0から始まるので、Pnum-1に修正
                    
                    // ページオブジェクトが有効か確認
                    if (page) {
                        // 重複しない名前を作成
                        var uniqueName = GetName;

                        // 名前が既存のブックマーク名と重複する限り番号を付けないようにします
                        if (existingNames[uniqueName]) {
                            // 同じ名前が既に存在する場合、ユニークな名前を付ける処理
                            alert("「" + uniqueName + "」は既に使用されています。ブックマーク名を重複しないようにしてください。");
                            continue;
                        }

                        // 新しい名前をリストに追加
                        existingNames[uniqueName] = true;

                        // ブックマークを追加
                        try {
                            var bm = doc.bookmarks.add(page);
                            bm.name = uniqueName; // 表の項目をブックマークの名前として設定
                        } catch (e) {
                            // エラーメッセージのデバッグ情報を追加
                            alert("ブックマークの作成に失敗しました: " + e.message + "\n" + e.line);
                            exit();
                        }
                    } else {
                        alert("指定されたページが見つかりません。");
                        exit(); // エラー終了
                    }
                } else {
                    alert("指定されたページ番号が範囲外です。ページ数を確認してください。");
                    exit(); // エラー終了
                }
            }

            alert("ブックマークを作成しました\nパネルを開いて確認して下さい");
        } else {
            alert("キャンセルしました");
            exit();
        }
    }
}
